/* Cmd SubModule  "bond.c"           -*- C -*- */

/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1998 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/

#define __bond_c__
#define __cmd_c_proc__
#define __subs_c_proc__
#define __stack_c_proc__
#define __shelplot_c_proc__
#define __syssub__

#include <math.h>
#include <ctype.h>
#include "def.h"

#define BOND_LIST_MAX_LINE 200

SV_Function void check_bond(void)
{
  int i;
  Double min_len=0.7,max_len=2.2;
  if (cmd_c==2) max_len=cmd_num[1];
  if (cmd_c==1 || cmd_c==2) min_len=cmd_num[0];
  for(i=0;i<n_bond;i++) {
    if ( BOND[i].len<min_len || BOND[i].len>max_len ) {
      cmd_c=2;
      cmd_atom[0]=SAT+BOND[i].atm[0];
      cmd_atom[1]=SAT+BOND[i].atm[1];
      del_bond();
      i--;
    }
  }
  refresh();
}

SV_Function void MakeBondMatrix(void)
{
  int i,j;
  BondMatrix=(int **)Malloc(sizeof(int *)*n_atom);
  BondMat=(int *)Malloc(sizeof(int)*n_atom*n_atom);
  BondMatrix2=(int **)Malloc(sizeof(int *)*n_atom);
  BondMat2=(int *)Malloc(sizeof(int)*n_atom*n_atom);
  nBondMatrix=(int *)Calloc(n_atom,sizeof(int));
  BondTag0=(int *)Calloc(n_atom+2,sizeof(int));
  BondTag1=(int *)Calloc(n_atom+2,sizeof(int));
  BondTag2=(int *)Calloc(n_satom+1,sizeof(int));
  dispgraph(NULL,-1);
  for(i=0;i<n_atom;i++) {
    BondMatrix[i]=BondMat+i*n_atom;
    BondMatrix2[i]=BondMat2+i*n_atom;
  }
  for(i=0;i<n_atom;i++){
    dispgraph("Make Bond Matrix",i*100/n_atom);
    for(j=i;j<n_atom;j++){
      ATOM_PARAM *atm1,*atm2;
      Float r2,l2;
      atm1=orgAtom+i;
      atm2=orgAtom+j;
      l2=get_length3(atm1,atm2);
      if (l2>16) continue;
      r2=atm1->radii+atm2->radii;
      r2*=r2;
      if (l2<r2*1.44 && 
          (l2>r2*0.49 ||  
           ((atm1->name[0]=='Q' || atm2->name[0]=='Q')  && l2>0.25))
          && (atm1->part==atm2->part || atm1->part==0 || atm2->part==0)) {
        BondMatrix[i][j]=1;
        BondMatrix2[i][nBondMatrix[i]]=j;
        nBondMatrix[i]++;
	if (i!=j) {
	  BondMatrix[j][i]=1;
	  BondMatrix2[j][nBondMatrix[j]]=i;
	  nBondMatrix[j]++;
	}
      } else {
        BondMatrix[i][j]=0;
        BondMatrix[j][i]=0;
      }
    }
  }
  for(i=0;i<n_satom;i++){
    BondTag0[SAT[i].atm]++;
  }
  BondTag1[0]=0;
  for(i=0;i<n_atom;i++){
    BondTag1[i+1]=BondTag1[i]+BondTag0[i];
  }
  for(i=0;i<=n_atom+1;i++){
    BondTag0[i]=BondTag1[i];
  }
  for(i=0;i<n_satom;i++){
    BondTag2[i]=BondTag1[SAT[i].atm]++;
  }
  dispgraph(NULL,-2);
}

SV_Function void FreeBondMatrix(void)
{
  Free(BondTag2);
  Free(BondTag1);
  Free(BondTag0);
  Free(nBondMatrix);
  Free(BondMat2);
  Free(BondMatrix2);
  Free(BondMat);
  Free(BondMatrix);
}

SV_Function void make_bond(void)
{
  int i,j,k;
  char buf[80];
  if (cmd_c==1 && chkinst(cmd_param[0],"SL","SELECT")) {
    cmd_param[0][0]=0;
    global_flag|=GLOBALFLAG_NOTREFRESH;
    cmd_c=2;
    for(i=0;i<atom_stack_top;i++){
      cmd_atom[0]=SAT+atom_stack[i];
      cmd_atom[1]=SAT+atom_stack[i+1];
      make_bond();
    }
    global_flag&=~GLOBALFLAG_NOTREFRESH;
    cmd_c=1;
    stack_clr();
    return;
  } else if (cmd_c==1 && chkinst(cmd_param[0],"PA","PAIR")) {
    cmd_param[0][0]=0;
    global_flag|=GLOBALFLAG_NOTREFRESH;
    cmd_c=2;
    for(i=atom_stack_top;i>0;i-=2){
      cmd_atom[0]=SAT+atom_stack[i-1];
      cmd_atom[1]=SAT+atom_stack[i];
      make_bond();
    }
    global_flag&=~GLOBALFLAG_NOTREFRESH;
    cmd_c=1;
    stack_clr();
    return;
  } else if (cmd_c==1 && chkinst(cmd_param[0],"AUTO","AUTO")) {
    MakeBondMatrix();
    dispgraph(NULL,-1);
    n_bond=0;
    for(i=0;i<n_satom;i++){
      S_AT *sat1,*sat2;
      sat1=SAT+i;
      gcolor(0);
      if ((i & 15)==0) {
        sprintf(buf,"Make Bond %4d%%",(int)((((Float)i)/n_satom)*100));
        dispgraph(buf,(int)((((Float)i)/n_satom)*100));
      }
      for(j=nBondMatrix[sat1->atm]-1;j>=0;j--){
        int b,e;
        b=BondTag0[BondMatrix2[sat1->atm][j]];
        e=BondTag1[BondMatrix2[sat1->atm][j]];
        for(k=b;k<e;k++){
          Double r,r2;
          sat2=SAT+BondTag2[k];
          if (BondMatrix[sat1->atm][sat2->atm]==0) continue;
          r=get_length(sat1,sat2);
          r2=sat1->radii+sat2->radii;
          if (((r>r2*0.7 && r<r2*1.2) ||
               (OrgAtom(i)->name[0]=='Q' && r<r2*1.2 && r>0.5))
              && (sat1->part==0 || sat2->part==0 ||
                  sat1->part==sat2->part)
              ) {
#ifdef BF4
		if (
            (OrgAtom(i)->name[0]!='F' || OrgAtom(sat2->num)->name[0]!='F') && 
            (OrgAtom(i)->name[0]!='H' || OrgAtom(sat2->num)->name[0]!='H')  
	            ) {
#endif
	      BOND_PARAM *b;
	      b=NewBond();
	      b->atm[0]=i;
	      b->atm[1]=sat2->num;
	      b->len=r;
#ifdef BF4
	    }
#endif
          }
        }
      }
    }
    dispgraph(NULL,-2);
    FreeBondMatrix();
    InsertCommandQueue("build #0");
    refresh();
  } else {
    S_AT *at1,*at2;
    for(j=0;j<cmd_c-1;j++){
      at1=cmd_atom[j];
      at2=cmd_atom[j+1];
      if (at1==NULL || at2==NULL) break;
      for(i=0;i<n_bond;i++) {
        if ((SAT+BOND[i].atm[0]==at1 && SAT+BOND[i].atm[1]==at2) 
            || (SAT+BOND[i].atm[0]==at2 && SAT+BOND[i].atm[1]==at1))
          {
            cmd_atom[0]=at1;
            cmd_atom[1]=at2;
            k=cmd_c;
            cmd_c=2;
            del_bond();
            cmd_c=k;
          }
      }
      {
        BOND_PARAM *b;
        b=NewBond();
        b->atm[0]=at1->num;
        b->atm[1]=at2->num;
        b->len=get_length(at1,at2);
      }
    }
    /*    dispgraph(NULL,-2);*/
    InsertCommandQueue("build #0");
    refresh();
  }
}

SV_Function void del_bond(void)
{
  int i,k;
  S_AT *at1,*at2;
  if (cmd_c==1 && chkinst(cmd_param[0],"SL","SELECT")) {
    cmd_param[0][0]=0;
    global_flag|=GLOBALFLAG_NOTREFRESH;
    cmd_c=2;
    for(i=0;i<atom_stack_top;i++) {
      cmd_atom[0]=SAT+atom_stack[i];
      cmd_atom[1]=SAT+atom_stack[i+1];
      del_bond();
    }
    global_flag&=~GLOBALFLAG_NOTREFRESH;
    cmd_c=1;
    stack_clr();
    return;
  } else if (cmd_c==1 && chkinst(cmd_param[0],"PA","PAIR")) {
    cmd_param[0][0]=0;
    global_flag|=GLOBALFLAG_NOTREFRESH;
    cmd_c=2;
    for(i=atom_stack_top;i>0;i-=2) {
      cmd_atom[0]=SAT+atom_stack[i];
      cmd_atom[1]=SAT+atom_stack[i-1];
      del_bond();
    }
    global_flag&=~GLOBALFLAG_NOTREFRESH;
    cmd_c=1;
    stack_clr();
    return;
  }
  for(k=0;k<cmd_c-1;k++){
    at1=cmd_atom[k];
    at2=cmd_atom[k+1];
    if (at1==NULL || at2==NULL) break;
    for(i=0;i<n_bond;i++) {
      if ((BOND[i].atm[0]==at1->num && BOND[i].atm[1]==at2->num) 
          || (BOND[i].atm[0]==at2->num && BOND[i].atm[1]==at1->num)) {
        BOND[i].atm[0]=BOND[n_bond-1].atm[0];
        BOND[i].atm[1]=BOND[n_bond-1].atm[1];
        BOND[i].len=BOND[n_bond-1].len;
        i--;
        n_bond--;
      }
    }
  }
  refresh();
}

SV_Function void reset_bond(void)
{
  S_AT *at1,*at2;
  int l;
  Double min_len;
  S_AT *x2;
  at1=cmd_atom[0];
  at2=cmd_atom[1];
  min_len=0;
  if (cmd_c==3) min_len=cmd_num[2];
  if (at1==NULL || at2==NULL) return;
  x2=get_length2(at1,orgAtom+at2->atm,min_len);
  at2->sym=x2->sym;
  for(l=0;l<3;l++) at2->sft[l]=x2->sft[l];
  sprintf(disp_str[0],"Bond Length ");
  sprintf(disp_str[1],"%-4s-%-4s %5.3f",cmd_param[0],cmd_param[1],x2->radii);
  disp_str[2][0]=0;
  refresh();
}

SV_Function void dup_atom(void)
{
  S_AT *at1,*at2;
  int i=0,l;
  Double min_len;
  S_AT *x2,*x3;
  at1=cmd_atom[i];
  at2=cmd_atom[i+1];
  min_len=0;
  if (cmd_c==3) min_len=cmd_num[i+2];
  if (at1==NULL || at2==NULL) return;
  x2=get_length2(at1,orgAtom+at2->atm,min_len);
  x2->atm=at2->atm;
  if (check_newatom(x2)==0) {
    cmderr("Already Exists.");
    return;
  }
  x3=NewSat();
  x3->sym=x2->sym;
  for(l=0;l<3;l++) x3->sft[l]=x2->sft[l];
  x3->atm=at2->atm;
  setup_symloc(x3,orgAtom);
  setup_atom(x3);
  sprintf(disp_str[0],"Bond Length ");
  sprintf(disp_str[1],"%-4s-%-4s %5.3f",cmd_param[0],cmd_param[1],x2->radii);
  disp_str[2][0]=0;
  refresh();
}

/*
   satp=NewSat();
   satp->atm=i;
   satp->sym=j;
   satp->sft[0]=x-shift[0];
   satp->sft[1]=y-shift[1];
   satp->sft[2]=z-shift[2];
   satp->view_flag=orgAtom[i].temp_num;
   satp->frag=orgAtom[i].frag;
   satp->part=orgAtom[i].part;
   setup_symloc(satp,orgAtom);
   flag=0;
   for(l=n_satom0;l<n_satom-1;l++){
   x0=satp->vloc[0]-SAT[l].vloc[0];
   y0=satp->vloc[1]-SAT[l].vloc[1];
   z0=satp->vloc[2]-SAT[l].vloc[2];
   if (x0*x0+y0*y0+z0*z0<0.01) {
   flag=1;break;
   }
   }
   if (flag==0){
   setup_atom(satp);
   } else {
   n_satom--;
*/

SV_LocalFunction void LocalAngleList(S_AT *atm)
{
  future();
}

SV_LocalFunction void LocalBondList(S_AT *at1,Double max_len) {
  S_AT sat,*satp,*satp2;
  ATOM_PARAM *at2;
  int i,j,atmn,x30,x31,x32;
  Double l0[3];
  Double x00,x01,x02  ,x10,x11,x12,  x20,x21,x22 ;
  Double x40,x41,x42  ,x50,x51,x52,  len;
  Double a0,a1,a2,a3,a4,a5;
  Double *lx;
  int bl[BOND_LIST_MAX_LINE][3],nb;
  Double bll[BOND_LIST_MAX_LINE];
  int n_line;
  nb=0;

/*  BackupStack();*/
  select_atom1(at1);
  sprintf(disp_str[0],"Bond Length List");
  sprintf(disp_str[1]," (<%5.2f to %s %d)",max_len,
	  OrgAtom(at1->atm)->name,EncodeAdc(at1));
  max_len*=max_len;
  n_line=2;
  lx=sym_exp(at1,orgAtom);
  for(atmn=0;atmn<n_atom;atmn++){
    at2=orgAtom+atmn;
    sat.atm=at2->num;
    a0=MAT[0][0];
    a1=MAT[1][0]; a2=MAT[1][1];
    a3=MAT[2][0]; a4=MAT[2][1]; a5=MAT[2][2];
/* x0? = (vx1,vy1,vz1) */
    x00=at1->vloc[0];
    x01=at1->vloc[1];
    x02=at1->vloc[2];
/* x1? = (x2,y2,z2) */
    x10=at2->loc[0];
    x11=at2->loc[1];
    x12=at2->loc[2];
    for(i=0;i<symnum;i++) {
/* x2? = (x2,y2,z2) + symmetry */
      x20=symm[i][0][0] + symm[i][0][1]*x10
	+ symm[i][0][2]*x11 + symm[i][0][3]*x12 ;
      x21=symm[i][1][0] + symm[i][1][1]*x10
	+ symm[i][1][2]*x11 + symm[i][1][3]*x12 ;
      x22=symm[i][2][0] + symm[i][2][1]*x10
	+ symm[i][2][2]*x11 + symm[i][2][3]*x12 ;

      if (fabs(floor(x20)-floor(lx[0]))>=1) {
	x20-=(l0[0]=floor(x20)-floor(lx[0]));
      }
      if (fabs(floor(x21)-floor(lx[1]))>=1) {
	x21-=(l0[1]=floor(x21)-floor(lx[1]));
      }
      if (fabs(floor(x22)-floor(lx[2]))>=1) {
	x22-=(l0[2]=floor(x22)-floor(lx[2]));
      }

/* x3? = (trans x,trans y,trans z) */
      for (x30=SEARCH_MIN;x30<=SEARCH_MAX;x30++) {
	for (x31=SEARCH_MIN;x31<=SEARCH_MAX;x31++) {
	  for (x32=SEARCH_MIN;x32<=SEARCH_MAX;x32++) {
/* x4? = (x2,y2,z2) + symmetry + translate */
	    x40=x20+x30;
	    x41=x21+x31;
	    x42=x22+x32;
/* x5? = (vx2,vy2,vz2) */
	    x50=a0*x40              -x00;
	    x51=a1*x40+a2*x41       -x01;
	    x52=a3*x40+a4*x41+a5*x42-x02;
	    len=x50*x50+x51*x51+x52*x52;
	    if (len<max_len){
	      bl[nb][0]=atmn;
/*	
              printf("%f %f %f\n",l0[0],l0[1],l0[2]);
	      bl[nb][1]=(x30+l0[0])*10000+(x31+l0[1])*1000
		+(x32+l0[2])*100+55500+i;
*/
	      bl[nb][1]=x30*10000+x31*1000+x32*100+55500+i;
	      bll[nb]=sqrt(len);
	      nb++;
	      if (nb>=BOND_LIST_MAX_LINE-10) goto xxx;
	    }
	  }
	}
      }
    }
  }
  xxx:
  for(i=0;i<nb;i++){
    for(j=0;j<n_satom;j++){
      if (bl[i][0]==SAT[j].atm) {
	if (EncodeAdc(SAT+j)==bl[i][1]){
	  bl[i][0]=-1;
	  bl[i][2]=j;
	  break;
	}
      }
    }
    if (j==n_satom){
printf("n_satom=%d\n",n_satom);      
      bl[i][2]=n_satom;
      satp2=DecodeAdc(bl[i][1]);
      satp=NewSat();
      satp->atm=bl[i][0];
      satp->sym=satp2->sym;
      satp->sft[0]=satp2->sft[0];
      satp->sft[1]=satp2->sft[1];
      satp->sft[2]=satp2->sft[2];
      satp->view_flag=0;
      satp->frag=-1;
      satp->part=orgAtom[i].part;
      setup_symloc(satp,orgAtom);
      setup_atom(satp);
    }
  }

  for(i=0;i<nb;i++){
    for(j=i+1;j<nb;j++){
      float ft;
      int it;
      int k;
      if (bll[i]>bll[j]){
	ft=bll[i];
	bll[i]=bll[j];
	bll[j]=ft;
	for(k=0;k<3;k++){
	  it=bl[i][k];
	  bl[i][k]=bl[j][k];
	  bl[j][k]=it;
	}
      }
    }
  }
  sprintf(disp_str[n_line++],"*** Intermolecular Contact ***");
  for(i=0;i<nb;i++){
    satp=SAT+bl[i][2];
    if (at1->frag!=satp->frag) {
      sprintf(disp_str[n_line++],"%-4s %5d %5.2f",OrgAtom(bl[i][2])->name,
	      bl[i][1],bll[i]);
printf("   %d\n",satp->num);      
      select_atom1(satp);
      satp->view_flag&=~FLAG_OMIT;
      if (n_line>=BOND_LIST_MAX_LINE-1) break;
    }
  }
  sprintf(disp_str[n_line++]," ");
  sprintf(disp_str[n_line++],"*** Intramolecular Contact ***");
  for(i=0;i<nb;i++){
    satp=SAT+bl[i][2];
    if (at1->frag==satp->frag) {
      sprintf(disp_str[n_line++],"%-4s %5d %5.2f",OrgAtom(bl[i][2])->name,
	      bl[i][1],bll[i]);
      select_atom1(satp);
      satp->view_flag&=~FLAG_OMIT;
      if (n_line>=BOND_LIST_MAX_LINE-1) break;
    }
  }
  disp_str[n_line][0]=0;
}

SV_Function void bond_list(void)
{
  S_AT *at1,*at2,*at3,*at4;
  if (cmd_c<1) return;
  if (chkinst(cmd_param[0],"SL","SELECT")) cmd_atom[0]=popstack();
  if (chkinst(cmd_param[0],"COLOR","COL")) {
    global_flag|=GLOBALFLAG_BOND_COLOR;
    refresh();
    return;
  } else if (chkinst(cmd_param[0],"MONO","MONOCHROME")) {
    global_flag&=~GLOBALFLAG_BOND_COLOR;
    refresh();
    return;
  }
  at1=cmd_atom[0];
  if (at1==NULL) return;
  switch(cmd_c){
  case 1:
    LocalAngleList(at1);
    disp_str[0][0]=0;
    break;
  case 2:
    at2=cmd_atom[1];
    if (at2!=NULL) {
      sprintf(disp_str[0],"Bond Length ");
      sprintf(disp_str[1],"%-4s-%-4s %5.3f",cmd_param[0],cmd_param[1],
              get_length(at1,at2));
      disp_str[2][0]=0;
    } else LocalBondList(at1,cmd_num[1]);
    break;
  case 3:
    at2=cmd_atom[1];
    at3=cmd_atom[2];
    if (at2==NULL || at3==NULL) return;
    sprintf(disp_str[0],"Bond Angle");
    sprintf(disp_str[1],"%-4s-%-4s-%-4s %6.2f",
            cmd_param[0],cmd_param[1],cmd_param[2],get_angle(at1,at2,at3));
    sprintf(disp_str[2],"%5.3f %5.3f",
            get_length(at1,at2),get_length(at2,at3));
    disp_str[3][0]=0;
    break;
  case 4:
    at2=cmd_atom[1];
    at3=cmd_atom[2];
    at4=cmd_atom[3];
    if (at2==NULL || at3==NULL || at4==NULL) return;
    sprintf(disp_str[0],"Torsion Angle");
    sprintf(disp_str[1],"%-4s-%-4s-%-4s-%-4s",
            cmd_param[0],cmd_param[1],cmd_param[2],cmd_param[3]);
    sprintf(disp_str[2],"%5.3f %5.3f %5.3f",
            get_length(at1,at2),get_length(at2,at3),get_length(at3,at4));
    sprintf(disp_str[3],"%6.2f %6.2f",
            get_angle(at1,at2,at3),get_angle(at2,at3,at4));
    sprintf(disp_str[4]," Torsion Angle %6.2f",get_torsion(at1,at2,at3,at4));
    disp_str[5][0]=0;
    break;
  }
  refresh();
}


SV_LocalVariable Double location0[3],vlocation0[3],*ftmp;
SV_LocalVariable Double radii0,bond_min;
SV_LocalVariable int snum,ssym,ssft[3],sli[3];

#if 0
SV_Function S_AT *findbondfirst(S_AT *atm,Double radii,Double min)
{
  int i;
  ftmp=sym_exp(atm,orgAtom);
  for(i=0;i<3;i++) {
    location0[i]=ftmp[i];
    sli[i]=0;
    ssft[i]=0;
  }
  bond_min=min*min;
  snum=-1;
  ssym=symnum;
  return findbondnext();
}

SV_Function S_AT *findbondnext(void)
{
  static Double loc[3];
  static Double vloc[3];
  static Double oloc[3];
  static Double tmp,sum;
  static int i;
  static S_AT sat;
  for(;;){
    if (++ssft[0]>1) {
      ssft[0]=0;
      if (++ssft[1]>1) {
        ssft[1]=0;
        if (++ssft[2]>1) {
          ssft[2]=0;
          if (++ssym>=symnum) {
            ssym=0;
            snum++;
            if (++snum>=n_atom) return NULL;
            for(i=0;i<3;i++){
              oloc[i]=orgAtom[snum].loc[i];
            }
            sum=(orgAtom[snum].radii+radii0)*(orgAtom[snum].radii+radii0);
          }
          for(i=0;i<3;i++){
            loc[i]=symm[ssym][i][0]+symm[ssym][i][1]*oloc[0]
              +symm[ssym][i][2]*oloc[1]+symm[ssym][i][2]*oloc[2];
            loc[i]+=(int)(location0[i]-loc[i]);
          }
        }
      }
    }
    vloc[0]=MAT[0][0]*loc[0]+MAT[1][0]*loc[1]+MAT[2][0]*loc[2];
    vloc[1]=MAT[0][1]*loc[0]+MAT[1][1]*loc[1]+MAT[2][1]*loc[2];
    vloc[2]=MAT[0][2]*loc[0]+MAT[1][2]*loc[1]+MAT[2][2]*loc[2];
    tmp=((vloc[0]-vlocation0[0])*(vloc[0]-vlocation0[0])
         +(vloc[1]-vlocation0[1])*(vloc[1]-vlocation0[1])
         +(vloc[2]-vlocation0[2])*(vloc[2]-vlocation0[2]));
    if (tmp>sum*0.7 && tmp<sum*1.2 && tmp>bond_min) {
      sat.sym=ssym;
      for(i=0;i<3;i++){
        sat.sft[i]=(int)(location0[i]-loc[i]);
      }
      sat.atm=orgAtom+snum;
      return &sat;
    }
  }
}


#endif




