/*  generate Charon file "charon.c"     -*- C -*-   */
/*  for Charon(teXsan version), Chemray(teXsan version), PDB(RasMol) */

/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1998 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/

#define __charon_c__
#define __cmd_c_proc__
#define __subs_c_proc__
#define __string_proc__

#include "def.h"
#include <ctype.h>

static int isChemRayMode=0;

SV_LocalFunction int c_ray_out_header(void)
{
  FILE *ch_fp;
  ch_fp=ffopen("charon.fil","w");
  if (ch_fp==NULL) {
    cmderr("Can't open charon.fil!");
    return 0;
  }
  fprintf(ch_fp,"%10.2f   %s\n",0.0,"Auto Bond Find");
  fprintf(ch_fp,"%10.2f   %s\n",20.0,"Figure Size in cm");
  fprintf(ch_fp,"%10.2f   %s\n",ZVIEW==0?1.0:0.95,"Perspective");
  fprintf(ch_fp,"%10.2f   %s\n",0.08,"Bond Radius");
  fprintf(ch_fp,"%10.2f   %s\n",0.15,"Atom Radius Scale");
  fprintf(ch_fp,"%10.2f   %s\n",1.0,"Atom and Bond Type");
  fprintf(ch_fp,"%10.2f   %s\n",0.0,"van der Waals Type");
  fprintf(ch_fp,"%10.2f   %s\n",2.0,"Bond Type");
  fprintf(ch_fp,"%10.2f   %s\n",1.0,"Atom Type");
  fprintf(ch_fp,"%10.2f   %s\n",
	  (global_flag & GLOBALFLAG_LABEL)?2.0:0.0,"Label Type");
  fprintf(ch_fp,"%10.2f   %s\n",20.0,"Font Size");
  fprintf(ch_fp,"%10.2f   %s\n",0.4,"Line Width");
  fprintf(ch_fp,"%10.2f   %s\n",0.9,"Back Ground Fill");
  fprintf(ch_fp,"%10.2f   %s\n",0.10,"Pluto Style Hatch");
  fprintf(ch_fp,"%10.2f   %s\n",5.00,"Pluto Style Hatch");
  fprintf(ch_fp,"%10.2f   %s\n",
	  (global_flag & GLOBALFLAG_STEREO)?5.0:0.0,"Stereo Separation");
  fprintf(ch_fp,"%10.2f   %s\n",3.0,"Stereo Rotation");
  fprintf(ch_fp,"%10.2f   %s\n",
	  (global_flag & GLOBALFLAG_CELL)?1.0:0.0,"Cell Outline Flag");
  fprintf(ch_fp,"%10.2f   %s\n",2.4,"Hydrogen Bond Length");
  fprintf(ch_fp,"%10.2f   %s\n",0.0,"Short Label");
  fprintf(ch_fp,"%10.2f   %s\n",0.0,"Line Drawing");
  fclose(ch_fp);
  return 1;
}

SV_LocalFunction int count_and_label_visible_atom(void)
{
  int i,nn_atom=0;
  for(i=0;i<n_satom;i++){
    SAT[i].temp_num=0;
    if ((SAT[i].view_flag & FLAG_OMIT)==0) {
      nn_atom++;
      SAT[i].temp_num=nn_atom;
    }	
  }
  return nn_atom;
}

SV_LocalFunction void c_ray_out(void)
{
  FILE *ch_fp;
  int i,nn_atom,nn_bond;
  S_AT *satp;
  char atm_name[8];
  double sc0=1.0;
  if (cmd_c>1) sc0=cmd_num[1];
  if (c_ray_out_header()==0) return;
  ch_fp=ffopen("c-ray.in","w");
  if (ch_fp==NULL) {
    cmderr("Can't open charon.fil!");
    return;
  }
  nn_atom=count_and_label_visible_atom();
  if (nn_atom==0) {
    cmderr("No visible atoms !");
    return;
  }
  fprintf(ch_fp,"%4d\n",nn_atom+((global_flag & GLOBALFLAG_CELL)?8:0));
  for(i=0;i<n_satom;i++){
    if ((SAT[i].view_flag & FLAG_OMIT)==0) {
      satp=SAT+i;
      if (satp->color<8) {
	sprintf(atm_name,"%s",orgAtom[satp->atm].name);
	if (isalpha(atm_name[1])) atm_name[1]=tolower(atm_name[1]);
      } else {
	atm_name[0]='q';
	atm_name[1]='a'+satp->color-8;
      }
      fprintf(ch_fp,
	"%-6s%10.5f%10.5f%10.5f%10.5f          %5.3f%5d%5d %c %2d%4.2f\n",
	      atm_name,satp->vloc[0]*sc0,satp->vloc[1]*sc0,satp->vloc[2]*sc0,
	      1.0,0.0,0,0,'_',satp->color,
	      sfac[orgAtom[satp->atm].atom_type].vdw);
    }	
  }
  if (global_flag & GLOBALFLAG_CELL) {
    for(i=0;i<8;i++) {
      satp=SYMCELL+i;
      if (isChemRayMode){
	fprintf(ch_fp,
		"Q%1d%1d%1d  %10.5f%10.5f%10.5f%10.5f          "
		"%5.3f%5d%5d %c %2d%4.2f\n",
		satp->sft[0],satp->sft[1],satp->sft[2],
		satp->vloc[0]*sc0,satp->vloc[1]*sc0,satp->vloc[2]*sc0,
		1.0,0.0,0,0,'_',0,satp->radii);
      } else {
	fprintf(ch_fp,
		"*%1d%1d%1d  %10.5f%10.5f%10.5f%10.5f          "
		"%5.3f%5d%5d %c %2d%4.2f\n",
		satp->sft[0],satp->sft[1],satp->sft[2],
		satp->vloc[0]*sc0,satp->vloc[1]*sc0,satp->vloc[2]*sc0,
		1.0,0.0,0,0,'_',0,satp->radii);
      }
    }
  }
  nn_bond=0;
  for(i=0;i<n_bond;i++){
    if ((SAT[BOND[i].atm[0]].view_flag & FLAG_OMIT)==0 &&
	(SAT[BOND[i].atm[1]].view_flag & FLAG_OMIT)==0 ) nn_bond++;
  }
  fprintf(ch_fp,"%4d\n",nn_bond+(((global_flag & GLOBALFLAG_CELL) && isChemRayMode)?12:0));
  for(i=0;i<n_bond;i++){
    if ((SAT[BOND[i].atm[0]].view_flag & FLAG_OMIT)==0 &&
	(SAT[BOND[i].atm[1]].view_flag & FLAG_OMIT)==0 ) {
      fprintf(ch_fp,"%5d%5d            1\n",
	      SAT[BOND[i].atm[0]].temp_num,SAT[BOND[i].atm[1]].temp_num);
    }
  }
  if (isChemRayMode && (global_flag & GLOBALFLAG_CELL)) {
    for(i=0;i<12;i++){
      fprintf(ch_fp,"%5d%5d            1\n",
	      nn_atom+BCELL[i].atm[0]+1,nn_atom+BCELL[i].atm[1]+1);
    }
  }
  fclose(ch_fp);
#ifdef CHARON_EXEC
  if (isChemRayMode){
    system("chemray");
    system("chemray2ppm chemray.wso");
    system("xv chemray.ppm");
  } else {
    system("charon");
    system("ghostview charon.ps");
  }
#endif
}

SV_Function void charonout(void)
{
  isChemRayMode=0;
  c_ray_out();
}

SV_Function void chemrayout(void)
{
  isChemRayMode=1;
  c_ray_out();
}

SV_Function void rasmolout(void) 
{
  FILE *fp;
  int nn_atom,i;
  S_AT *satp;
  Double RMAT[3][3];
  char atm_name[80];
  fp=fopen("sv.pdb","w");
  if (fp==NULL) cmderr("Can't open sv.pdb for temporary");
  nn_atom=count_and_label_visible_atom();
  if (nn_atom==0) {
    cmderr("No visible atoms !");
    return;
  }
  fprintf(fp,"COMPND SV-Output Temp\n");
  fprintf(fp,"REMARK This file was made automatically by 'SV'\n");

  /**** Fractional Coordinate **********/
  inv_matrix(MAT,RMAT);
  fprintf(fp,"CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f P 1        1\n",
	  cell[0],cell[1],cell[2],
#ifdef RASMOL_BUG
	  180.0-cell[3]/PI*180.0,180.0-cell[4]/PI*180.0,
	  180.0-cell[5]/PI*180.0
#else
	  cell[3]/PI*180.0,cell[4]/PI*180.0,cell[5]/PI*180.0
#endif
	  );
/*
  fprintf(fp,"SCALE1    %10.6f%10.6f%10.6f%10.6f\n",
	  RMAT[0][0],RMAT[0][1],RMAT[0][2],0.0);
  fprintf(fp,"SCALE2    %10.6f%10.6f%10.6f%10.6f\n",
	  RMAT[1][0],RMAT[1][1],RMAT[1][2],0.0);
  fprintf(fp,"SCALE3    %10.6f%10.6f%10.6f%10.6f\n",
	  RMAT[2][0],RMAT[2][1],RMAT[2][2],0.0);
  fprintf(fp,"SCALE1    %10.6f%10.6f%10.6f%10.6f\n",
	  RMAT[0][2],RMAT[0][0],RMAT[0][1],0.0);
  fprintf(fp,"SCALE2    %10.6f%10.6f%10.6f%10.6f\n",
	  RMAT[1][2],RMAT[1][0],RMAT[1][1],0.0);
  fprintf(fp,"SCALE3    %10.6f%10.6f%10.6f%10.6f\n",
	  RMAT[2][2],RMAT[2][0],RMAT[2][1],0.0);
*/
  /*************************************/



  for(i=0;i<n_satom;i++){
    if ((SAT[i].view_flag & FLAG_OMIT)==0) {
      satp=SAT+i;
      if (satp->color<8) {
	sprintf(atm_name,"%s",OrgAtom(i)->name);
	/*
	  if (global_flag  & GLOBALFLAG_LONGLABEL) {
	  char tmp_str[10];
	  sprintf(tmp_str,"_%d",EncodeAdc(satp));
	  strcat(atm_name,tmp_str);
	  }	
	  if (global_flag  & GLOBALFLAG_DISPAFIX) {
	  char tmp_str[6];
	  sprintf(tmp_str,"*%d",OrgAtom(i)->afix);
	  strcat(atm_name,tmp_str);
	  }
	  */
	if (isalpha(atm_name[1])) atm_name[1]=tolower(atm_name[1]);
      } else {
	atm_name[0]='Q';
	atm_name[1]='A'+satp->color-8;
      }
      fprintf(fp,
	      "ATOM%7d  %-5s%8d    %8.3f%8.3f%8.3f%6.2f%6.2f\n",
	      satp->temp_num,atm_name,1,
	      satp->vloc[0],satp->vloc[1],satp->vloc[2],
	      1.0, 0.0);
    }	
  }
  /*
  if (global_flag & GLOBALFLAG_CELL) {
    for(i=0;i<8;i++) {
      satp=SYMCELL+i;
      fprintf(fp,
	      "ATOM%7d  %-5s%8d    %8.3f%8.3f%8.3f%6.2f%6.2f\n",
	      nn_atom+i+1,"X",1,
	      satp->vloc[0],satp->vloc[1],satp->vloc[2],
	      1.0, 0.0);
    }
    nn_atom+=8;
  }
  */
  fprintf(fp,"TER %7d\n",nn_atom+1);
  for(i=0;i<n_bond;i++){
    if ((SAT[BOND[i].atm[0]].view_flag & FLAG_OMIT)==0 &&
	(SAT[BOND[i].atm[1]].view_flag & FLAG_OMIT)==0 ) {
      fprintf(fp,"CONECT%5d%5d\n",
	      SAT[BOND[i].atm[0]].temp_num,SAT[BOND[i].atm[1]].temp_num);
    }
  }
  /*  if (global_flag & GLOBALFLAG_CELL) {
    nn_atom-=8;
    for(i=0;i<12;i++){
      fprintf(fp,"CONECT%5d%5d\n",
	      nn_atom+BCELL[i].atm[0]+1,nn_atom+BCELL[i].atm[1]+1);
    }
  }
  */
  fprintf(fp,"END\n");
  fclose(fp);
#ifdef RASMOL_EXEC
  system("rasmol sv.pdb");
#endif
}
