/* Macro / typedef / prototype module "def.h"   -*- C -*- */
/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1999 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/
/* $Id: def.h 131 2006-02-14 08:14:55Z tnemoto $ */

#ifndef __def_h__

#ifndef __STDIO_H__
#include <stdio.h>
#endif

#ifndef __STDLIB_H__
#include <stdlib.h>
#endif

#if defined(HAVE_CONFIG_H) && !defined(__CONFIG_H__)
#include "../config.h"
#endif


#ifdef __string_proc__
  #ifdef __UNIX__
    #ifdef HAVE_STRING_H
      #include <string.h>
    #else
      #include <strings.h>
    #endif 
  #else 
    #include <string.h>
  #endif
#endif 

#ifdef __UNIX__
#define FileDelimitor '/'
#else
#define FileDelimitor '\\'
#endif

#ifndef PI
#define PI 3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679
#endif

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
#define OK 1
#define ERR 0

typedef double Double;
typedef signed char Mini;
typedef unsigned char Byte;
typedef signed char Bool;

#ifdef __MSDOS__
typedef long int Long;
typedef double Float;
#else
typedef int Long;
typedef double Float;
#endif

#define MAX_PATH_LENGTH 1024
#define MAX_BOND_PAR_ATOM 20

#define ATOM_BLOCK_SIZE 1024
#define BOND_BLOCK_SIZE 2048
#define SVX_BLOCK_SIZE 128
#define STRING_BLOCK_SIZE 8192
#define STACK_BLOCK_SIZE 1024
#define MAX_SYMM 256
#define MAX_SFAC 20
#define MAX_ILINES 5000
#define MAX_IARCS 15000
#define MAX_IORGS 20
#define MAX_IRVMTS 20
#define MAX_CMD_PARAM 100
#define COLOR_NORMAL 0
#define COLOR_REVERSEL 0

#define FLAG_NOLABEL    1
#define FLAG_SEL        2
#define FLAG_OMIT       4
#define FLAG_HIGHLIGHT  8
#define FLAG_TEMP 16
#define FLAG_NOBOND 32
#define GLOBALFLAG_PLOT             1
#define GLOBALFLAG_LABEL            2
#define GLOBALFLAG_CELL             4
#define GLOBALFLAG_BONDLEN          8
#define GLOBALFLAG_NOTREFRESH       16
#define GLOBALFLAG_TMP_LABEL        32
#define GLOBALFLAG_STEREO           64
#define GLOBALFLAG_IGRAPH           128
#define GLOBALFLAG_GC               256
#define GLOBALFLAG_LONGLABEL        512
#define GLOBALFLAG_DISPAFIX         1024
#define GLOBALFLAG_DISPDFIX         2048
#define GLOBALFLAG_LABELELEMENT     4096
#define GLOBALFLAG_ATOM_CIRCLE      8192
#define GLOBALFLAG_ATOM_BLOCK       16384
#define GLOBALFLAG_BOND_COLOR       32768
#define GLOBALFLAG_COLORPLOT        65536
#define cBufSize 1024

#define OrgAtom(num) (orgAtom+SAT[num].atm)

typedef struct {
  char name[5];
  int temp_num;
  int atom_type;
  Float radii;
  Double loc[3];
  Double uij[6];
/*  0=U11 1=U22 2=U33 3=U23 4=U13 5=U12 */
  Float occ;
  Float peak_h;
  int afix;
  int frag;
  int part;
  int warn;
  int num;
} ATOM_PARAM;

typedef struct {
  int atm;                      /* Original Atom */
  int sym,sft[3];               /* Symmetry Expand */
  int view_flag;
  int part;          
  int frag;                     /* Flag, Fragment */
  Float radii;
  Mini color;                   /* Atom Color */
  Float vloc[3];                /* Cartesian Coordinate */
  Float axy[3];                 /* Projection */
  int temp_num;
  int num;
} S_AT;

typedef struct {
  int atm[2];
  Float len;
  int num;
} BOND_PARAM;

typedef struct {
        char name[3];
        Double rad;
        int col;
        Double vdw;
} VDW;

typedef struct {
  Float vloc[3];
  Float r;
  int st,ed;
  int col;
  int matnum;
  int orgnum;
} IGraph_Arcs;

typedef struct {
  Float vloc[3];
  Float vloc2[3];
  int col;
  int matnum;
  int orgnum;
} IGraph_Lines;

typedef struct {
  char CheckType;
  int nKeyword;
  int Keyword;
  int KeywordLength;
  int Command;
} FileTypeCheck_T; 

#define stacktop (SAT+atom_stack[atom_stack_top])

#ifdef __main_c_proc__
#ifdef __UNIX__
/* #define __xtmain_c_proc__ */
#endif
#ifdef __MSWIN__
#define __winmain_c_proc__
#endif
#endif

#if defined(__syssub__) || defined (__subs_c_proc__)
#ifdef __UNIX__
#define __unixsub_c_proc__
#endif
#ifdef __MSWIN__
#define __winsub_c_proc__
#endif
#endif

#ifndef Xt
#undef __xtmain_c_proc__ 
#undef __xtcalls_c_proc__
#undef __xtmenus__
#endif

#if defined(Xt) && ((defined(__xtmain_c__) || defined(__xtmain_c_proc__) \
   || defined(__xtcalls_c__) || defined(__xtcalls_c_proc__) \
   || defined (__xtmenus__)))
#include "../unix/xt.h"
#endif

#if defined(__shelplot_c_proc__) || defined(__shelplot_c__)
#ifdef __MSWIN__
#define __plotwin_c_proc__
#else
#define __plotxt_c_proc__
#endif
#endif

#define SV_GlobalFunction extern
#define SV_GlobalVariable extern
#define SV_Function extern
#define SV_Variable extern
#define SV_LocalFunction static
#define SV_LocalVariable extern
#include "funcs.h"
#include "vars.h"
#undef SV_GlobalFunction
#undef SV_GlobalVariable
#undef SV_LocalFunction
#undef SV_Function
#undef SV_LocalVariable
#undef SV_Variable
#define SV_GlobalFunction
#define SV_GlobalVariable
#define SV_Function
#define SV_Variable
#define SV_LocalFunction static
#define SV_LocalVariable

#include "nowarn.h"  /* for sun with gcc */

#ifdef DEBUG
#define debug(str) PutMessage1(str)
#else
#define debug(str)
#endif

#if defined(CAVITY_EXEC) && !defined(HAVE_WAIT3)
#ifdef CAVITY_DISP_PROGRESS
#undef CAVITY_DISP_PROGRESS
#warning CAVITY_DISP_PROGRESS disabled
#endif
#endif

#define __def_h__
#endif
