/*  generate Ortep input file    "ortep.c"      -*- C -*-   */

/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1998 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/

#define __ortep_c__
#define __cmd_c_proc__
#define __subs_c_proc__

#include "def.h"
#include <ctype.h>

SV_LocalVariable int SV_OrtepSelectCode;
SV_LocalVariable int SV_OrtepAnisCode;
SV_LocalVariable int SV_OrtepIsoCode;
SV_LocalVariable int SV_OrtepHydrogenCode;
SV_LocalVariable Float SV_OrtepAtomRadii;
SV_LocalVariable Float SV_OrtepHydrogenRadii;

SV_LocalFunction int OrtepAtomCode(int atom_num,int viewFlag) {
  int code;
  code=SV_OrtepAnisCode;
  if (orgAtom[atom_num].uij[1]==0) code=SV_OrtepIsoCode;
  if (orgAtom[atom_num].name[0]=='H' && !isalpha(orgAtom[atom_num].name[1])) {
    code=SV_OrtepHydrogenCode;
  }
  if ((viewFlag & FLAG_SEL)!=0) {
    code=SV_OrtepSelectCode;
  }
  if ((viewFlag & FLAG_NOLABEL)!=0 || 
      (global_flag & GLOBALFLAG_LABEL)==0) code=-code;
  return code;
}

SV_LocalFunction void OrtepPlotAtom(int version,FILE *or_fp) {
  int code2=-1;
  int start=1;
  int last;
  int i;
  int *viewFlag;
  Float labelSize=0.1,lbSize;
  viewFlag=(int *)Calloc(n_atom,sizeof(int));
  if (version==2) labelSize=18.0;  /* for Ortep for Sun */
  lbSize=labelSize;
  for(i=0;i<n_satom;i++){
    viewFlag[SAT[i].atm]|=SAT[i].view_flag;
  }
  for(i=0;i<n_atom;i++){
    if (orgAtom[i].temp_num!=0) {
      int code;
      code=OrtepAtomCode(i,viewFlag[i]);
      if (code2!=code) {
	if (code2!=-1) {
	  if (code2<0) {
	    lbSize=0;
	    code2=-code2;
	  } else lbSize=labelSize;
	  fprintf(or_fp,
		  "  1   %3d                                    %9.3f\n",
		  code2, lbSize  );
	  fprintf(or_fp,
		  "                           %9d%9d\n",start,last);
	}
	code2=code;
	start=orgAtom[i].temp_num;
      }
      last=orgAtom[i].temp_num;
    }
  }
  if (code2!=-1) {
    if (code2<0) {
      lbSize=0;
      code2=-code2;
    }
    fprintf(or_fp,
	    "  1   %3d                                    %9.3f\n",
	    code2, lbSize  );
    fprintf(or_fp,
	    "                           %9d%9d\n",start,last);
  }
  Free(viewFlag);
}

SV_LocalFunction void ortep_file_generate(int version)
{
  FILE *or_fp;
  int i,j,k,count,nn_atom;
  S_AT *at1,*at2;
  ATOM_PARAM *at;
  Double iMAT[3][3];
  Double iVMT[3][3];
  Double imatrix[3][3];
  Double tmp1,tmp2,cg[3];

  SV_OrtepSelectCode=711;
  SV_OrtepAnisCode=714;
  SV_OrtepIsoCode=714;
  SV_OrtepHydrogenCode=714;
  SV_OrtepAtomRadii=0;
  SV_OrtepHydrogenRadii=0;
  

  if (cmd_c>=1) SV_OrtepAtomRadii=cmd_num[0];
  if (cmd_c>=2) SV_OrtepHydrogenRadii=cmd_num[1];
  if (cmd_c>=3 && cmd_int[2]>710 && cmd_int[2]<716) 
    SV_OrtepSelectCode=cmd_int[2];
  if (cmd_c>=4 && cmd_int[3]>710 && cmd_int[3]<716) 
    SV_OrtepAnisCode=cmd_int[3];
  if (cmd_c>=5 && cmd_int[4]>710 && cmd_int[4]<716) 
    SV_OrtepIsoCode=cmd_int[4];
  if (cmd_c>=6 && cmd_int[5]>710 && cmd_int[5]<716) 
    SV_OrtepHydrogenCode=cmd_int[5];

  inv_matrix(MAT,iMAT);
  inv_matrix(VMT,iVMT);
  for(i=0;i<3;i++){
    for(j=0;j<3;j++){
      imatrix[i][j]=0;
      for(k=0;k<3;k++){
	imatrix[i][j]+=iMAT[i][k]*iVMT[k][j];
      }
    }
  }
  for(i=0;i<3;i++){
/*     cg[i]=imatrix[i][0]*avg[0]+imatrix[i][1]*avg[1]+imatrix[i][2]*avg[2];*/
/*     cg[i]=avg[i];*/
     cg[i]=0;
  };
  for(i=0;i<n_atom;i++) {
    orgAtom[i].temp_num=0;
  };
  for(i=0;i<n_satom;i++) {
    if ((SAT[i].view_flag & FLAG_OMIT)==0) OrgAtom(i)->temp_num=1;
  };
  j=1;
  for(i=0;i<n_atom;i++) {
    if (orgAtom[i].temp_num!=0) orgAtom[i].temp_num=j++;
  };
  nn_atom=j-1;
  if (version==2) {
    or_fp=ffopen("ortep.in","w");
    if (or_fp==NULL) {
      cmderr("Can not open 'ortep.in'");
      return;
    }
  } else {
    or_fp=ffopen("TEP.IN","w");
    if (or_fp==NULL) {
      cmderr("Can not open 'TEP.IN'");
      return;
    }
  }
  fprintf(or_fp,"Ortep\n"); /* Title */
  if (version==2) {
    fprintf(or_fp,"%9.6f%9.6f%9.6f%9.6f%9.6f%9.6f\n",
	    cell[0],cell[1],cell[2],
	    cell[3]/PI*180,cell[4]/PI*180,cell[5]/PI*180);
  } else {
    fprintf(or_fp,"0%8.5f%9.6f%9.6f%9.6f%9.6f%9.6f\n",
	    cell[0],cell[1],cell[2],
	    cell[3]/PI*180,cell[4]/PI*180,cell[5]/PI*180);
  }
  for(i=0;i<symnum;i++){
    fprintf(or_fp,"%c%14.10f%3.0f%3.0f%3.0f%15.10f%3.0f%3.0f%3.0f%15.10f%3.0f"
	    "%3.0f%3.0f\n",i==symnum-1?'1':' ',
	    symm[i][0][0],symm[i][0][1],symm[i][0][2],symm[i][0][3],
	    symm[i][1][0],symm[i][1][1],symm[i][1][2],symm[i][1][3],
	    symm[i][2][0],symm[i][2][1],symm[i][2][2],symm[i][2][3]);
  }
  for(i=0;i<n_atom;i++) {
    at=orgAtom+i;
    if (at->temp_num!=0) {
      int HFlag=0;
      fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	      at->name,0.0,0.0,at->loc[0],at->loc[1],at->loc[2],0.0);
      tmp1=at->uij[0];
      tmp2=8.0;  /* ANIS */
      if (at->name[0]=='H' && !isalpha(at->name[1])){
	HFlag=1;
      }
      if (tmp1==0) { 
	/* Peak, etc */
	tmp1=0.1;
	tmp2=6.0; /* ISO */
        fprintf(or_fp," %8.5f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	      tmp1,0.0,0.0,0.0,0.0,0.0,tmp2);
      } else if (SV_OrtepAtomRadii!=0 || 
		 (HFlag && SV_OrtepHydrogenRadii!=0) ||
		 at->uij[1]==0) {
	/* Iso */
	if (HFlag){
	  tmp1=(SV_OrtepHydrogenRadii==0?tmp1*8*PI*PI:SV_OrtepHydrogenRadii);
	} else {
	  tmp1=(SV_OrtepAtomRadii==0?tmp1*8*PI*PI:SV_OrtepAtomRadii);
	}
	tmp2=6.0; /* Iso */
        fprintf(or_fp," %8.5f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	      tmp1,0.0,0.0,0.0,0.0,0.0,tmp2);
      } else {
	/* Anis */
        fprintf(or_fp," %8.5f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	      tmp1,
	      at->uij[1],at->uij[2],at->uij[5],at->uij[4],at->uij[3],
	      tmp2);
      }
    };
  }
  fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	  " ",0.0,0.0,0.0,0.0,0.0,0.0);  /* origin */
  fprintf(or_fp,"%9.6f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	  0.01,0.0,0.0,0.0,0.0,0.0,7.0);
  fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	  "xaxis",0.0,0.0,imatrix[0][0],imatrix[1][0],imatrix[2][0],0.0);
  fprintf(or_fp,"%9.6f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	  0.01,0.0,0.0,0.0,0.0,0.0,7.0);
  fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	  "yaxis",0.0,0.0,imatrix[0][1],imatrix[1][1],imatrix[2][1],0.0);
  fprintf(or_fp,"%9.6f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	  0.01,0.0,0.0,0.0,0.0,0.0,7.0);
  fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	  "o",0.0,0.0,imatrix[0][1],imatrix[1][1],imatrix[2][1],0.0);
  fprintf(or_fp,"%9.6f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	  0.01,0.0,0.0,0.0,0.0,0.0,7.0);
  fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	  "a",0.0,0.0,1.0,0.0,0.0,0.0);
  fprintf(or_fp,"%9.6f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	  0.01,0.0,0.0,0.0,0.0,0.0,7.0);
  fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	  "b",0.0,0.0,0.0,1.0,0.0,0.0);
  fprintf(or_fp,"%9.6f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	  0.01,0.0,0.0,0.0,0.0,0.0,7.0);
  fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	  "c",0.0,0.0,0.0,0.0,1.0,0.0);
  fprintf(or_fp,"%9.6f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	  0.01,0.0,0.0,0.0,0.0,0.0,7.0);
  fprintf(or_fp,"%-6s   %9.6f%9.6f%9.6f%9.6f%9.6f%9.0f\n",
	  "cgrav",0.0,0.0,cg[0],cg[1],cg[2],0.0);
  fprintf(or_fp,"1%8.6f%9.6f%9.6f%9.6f%9.6f%9.6f       %2.0f\n",
	  0.1,0.0,0.0,0.0,0.0,0.0,7.0);
  fprintf(or_fp,"      201\n");
  if (version==2) {
    fprintf(or_fp,"      301    600.0    600.0    0.000    30.00\n");
  } else {
    fprintf(or_fp,"      301     10.0     10.0    0.000     0.80\n");
  }
  /*  for   ORTEP for Sun   */
  fprintf(or_fp,"      303    0.0\n");
  count=0;
  for(i=0;i<n_satom;i++) {
    at1=SAT+i;
    if ((at1->view_flag & FLAG_OMIT)==0) {
      if (count==0) fprintf(or_fp,"      401");
      fprintf(or_fp,"%9d",
	   ((Long)100000)*(orgAtom[at1->atm].temp_num)+EncodeAdc(at1)+1);
      if (++count==7) { fprintf(or_fp,"\n"); count=0;}
    }
  }
  if (count!=0) fprintf(or_fp,"\n");
  if ((global_flag & GLOBALFLAG_CELL)!=0) {
    fprintf(or_fp,"      401%4d55501%4d55601%4d56501%4d56601\n",
	    nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1);
    fprintf(or_fp,"      401%4d65501%4d65601%4d66501%4d66601\n",
	    nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1);
  }
  fprintf(or_fp,
   "      501%4d55501%4d55501%4d55501%4d55501%4d55501                 1\n",
	  nn_atom+8,nn_atom+1,nn_atom+2,nn_atom+1,nn_atom+3);
         /*  center   o        x       o         y  */

  fprintf(or_fp,"      604        0        0        0     1.54\n");
  /* 50% */
/*   fprintf(or_fp,"      611        0        0      0.9\n");*/
  if (version<=2) {
    fprintf(or_fp,"      511        2\n");
  } else {
    fprintf(or_fp,"     1001        0\n");
  }
  OrtepPlotAtom(version,or_fp);
  count=0;
  for(i=0;i<n_bond;i++) {
    at1=SAT+BOND[i].atm[0];
    at2=SAT+BOND[i].atm[1];
    if ( (at1->view_flag & FLAG_OMIT) ==0 &&
	 (at2->view_flag & FLAG_OMIT) ==0 ) {
      if (count==0) {
	fprintf(or_fp,"  2   811");
      };
      fprintf(or_fp,"%9d%9d",
	      EncodeAdc(at1)+1+100000*(orgAtom[at1->atm].temp_num),
	      EncodeAdc(at2)+1+100000*(orgAtom[at2->atm].temp_num));
      if (++count==3) {
	fprintf(or_fp,"\n                       5              0.07\n");
	count=0;
      }
    }
  }
  if (count!=0) {
    fprintf(or_fp,"\n                       5              0.07\n");
  }
  if ((global_flag & GLOBALFLAG_CELL)!=0) {
    fprintf(or_fp,"  2   811%4d55501%4d65501%4d55501%4d56501%4d55501%4d55601"
        "\n                       1              0.01\n",
	nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1);
    fprintf(or_fp,"  2   811%4d56501%4d66501%4d65501%4d66501%4d65501%4d65601"
        "\n                       1              0.01\n",
	nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1);
    fprintf(or_fp,"  2   811%4d56501%4d56601%4d66501%4d66601%4d65601%4d66601"
        "\n                       1              0.01\n",
	nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1);
    fprintf(or_fp,"  2   811%4d55601%4d65601%4d55601%4d56601%4d56601%4d66601"
        "\n                       1              0.01\n",
	nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1,nn_atom+1);
  }
  if (version!=2){
    fprintf(or_fp,"      202\n");
  }
  fprintf(or_fp,"       -1\n");
  fclose(or_fp);
#ifdef ORTEP_EXEC
  if (version==2) {
    system("ortep ortep.in");
  } else {
    system("ortep3 TEP.IN");
  }
#endif
}

SV_Function void ortout(void)
{
  ortep_file_generate(2);
}

SV_Function void ort3out(void)
{
  ortep_file_generate(3);
}

