/*  generate Povray file "povray.c"     -*- C -*-   */
/*  for POVRAY */

/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1998 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/

#define __povray_c__
#define __cmd_c_proc__
#define __subs_c_proc__
#define __string_proc__

#include "def.h"
#include <ctype.h>

SV_LocalFunction int povray_out_header(FILE *fp)
{
  fprintf(fp,
"#version 3.5;\n"
"#include \"colors.inc\"\n"
"#include \"functions.inc\"\n"
"#include \"shapes.inc\"\n"

"global_settings {\n"
"//        assumed_gamma 1\n"
"}\n"

"#declare CamLoc = < 0, 0, -20>;\n"

"camera {\n"
"        location CamLoc\n"
"        look_at < 0, 0, 0>\n"
"        angle 45\n"
"}\n"

"light_source {CamLoc color White*0.35}\n"
"light_source {<-50, 150,-75> color White}\n"
"background { color White*0.3 }\n"
	  );
  return 1;
}

SV_LocalFunction int count_and_label_visible_atom(void)
{
  int i,nn_atom=0;
  for(i=0;i<n_satom;i++){
    SAT[i].temp_num=0;
    if ((SAT[i].view_flag & FLAG_OMIT)==0) {
      nn_atom++;
      SAT[i].temp_num=nn_atom;
    }	
  }
  return nn_atom;
}

SV_Function void povrayout(void) 
{
  FILE *fp;
  int nn_atom,i;
  S_AT *satp;
  Double RMAT[3][3];
  char atm_name[80];
  static Double RGBT[][3]={
    /* For Normal */
    {0.1,0.1,0.1},
    {0.1,0.1,1.0},
    {1.0,0.1,0.1},
    {1.0,0.1,1.0},
    {0.1,0.7,0.1},
    {0.1,1.0,1.0},
    {1.0,1.0,0.1},
    {1.0,1.0,1.0},
  };
  fp=fopen("sv.pov","w");
  if (fp==NULL) cmderr("Can't open sv.pov for temporary");
  nn_atom=count_and_label_visible_atom();
  if (nn_atom==0) {
    cmderr("No visible atoms !");
    return;
  }
  
  fprintf(fp,"// This file was made automatically by 'SV'\n");
  povray_out_header(fp);

  /**** Fractional Coordinate **********/
  inv_matrix(MAT,RMAT);

  for(i=0;i<n_satom;i++){
    if ((SAT[i].view_flag & FLAG_OMIT)==0) {
      int color;
      satp=SAT+i;
      color=satp->color & 7;
      fprintf(fp,
	      "sphere { \n"
	      "  < %8.3f, %8.3f, %8.3f >, %6.2f \n"
	      "  texture {\n"
	      "    pigment { color rgb <%5.2f, %5.2f, %5.2f> }\n"
	      "    finish {\n"
	      "//      ambient 0.3 \n"
	      "      metallic\n"
	      "//      reflection 0.2\n"
	      "      specular 1\n"
	      "    }\n"
	      "  }\n"
	      "}\n",
	      satp->vloc[0]-avg[0],satp->vloc[1]-avg[1],
	      -(satp->vloc[2]-avg[2]), 
	      sfac[orgAtom[satp->atm].atom_type-1].vdw,
	      RGBT[color][0],RGBT[color][1],RGBT[color][2]);
    }	
  }
#if 0
  for(i=0;i<n_bond;i++){
    if ((SAT[BOND[i].atm[0]].view_flag & FLAG_OMIT)==0 &&
	(SAT[BOND[i].atm[1]].view_flag & FLAG_OMIT)==0 ) {
      fprintf(fp,"CONECT%5d%5d\n",
	      SAT[BOND[i].atm[0]].temp_num,SAT[BOND[i].atm[1]].temp_num);
    }
  }
#endif
  fclose(fp);
#ifdef POVRAY_EXEC
  system("povray sv.pov");
#endif
}
