/* File Read Module   "readf.c"    -*- C -*-  */

/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1998 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/

#define __readf_c__
#define __subs_c_proc__
#define __atom_c_proc__
#define __cmd_c_proc__
#define __string_proc__
#include <math.h>
#include "def.h"

SV_LocalFunction void DecodeSymmetry(void)
{
  int i,j,k,n;
  Double a;

  n=abs(LATT);
  switch (n) {
  case 1:
    break;
  case 2:
    for (i=0;i<symnum;i++) {
      for(j=0;j<3;j++) {
        for (k=0;k<4;k++) {
          if (k==0) a=0.5; else a=0;
          symm[i+symnum][j][k]=symm[i][j][k]+a;
        }
      }
    }
    symnum+=symnum;
    break;
  case 3:
    for (i=0;i<symnum;i++) {
      for(j=0;j<3;j++) {
        for (k=0;k<4;k++) {
          symm[i+symnum][j][k]=symm[i][j][k];
          symm[i+symnum*2][j][k]=symm[i][j][k];
        }
      }
      symm[i+symnum][0][0]+=2.0/3;
      symm[i+symnum][1][0]+=1.0/3;
      symm[i+symnum][2][0]+=1.0/3;
      symm[i+symnum*2][0][0]+=1.0/3;
      symm[i+symnum*2][1][0]+=2.0/3;
      symm[i+symnum*2][2][0]+=2.0/3;
    }
    symnum*=3;
    break;
  case 4:
    for (i=0;i<symnum;i++) {
      for(j=0;j<3;j++) {
        for (k=0;k<4;k++) {
          if (k==0 && j!=0 ) a=0.5; else a=0;
          symm[i+symnum][j][k]=symm[i][j][k]+a;
        }
      }
    }
    for (i=0;i<symnum;i++) {
      for(j=0;j<3;j++) {
        for (k=0;k<4;k++) {
          if (k==0 && j!=1) a=0.5; else a=0;
          symm[i+symnum*2][j][k]=symm[i][j][k]+a;
        }
      }
    }
    for (i=0;i<symnum;i++) {
      for(j=0;j<3;j++) {
        for (k=0;k<4;k++) {
          if (k==0 && j!=2) a=0.5; else a=0;
          symm[i+symnum*3][j][k]=symm[i][j][k]+a;
        }
      }
    }
    symnum*=4;
    break;
  case 5:
  case 6:
  case 7:
    for (i=0;i<symnum;i++) {
      for(j=0;j<3;j++) {
        for (k=0;k<4;k++) {
          if (k==0 && j!=n-5 ) a=0.5; else a=0;
          symm[i+symnum][j][k]=symm[i][j][k]+a;
        }
      }
    }
    symnum*=2;
    break;
  }
  if (LATT>0) {
    for (i=0;i<symnum;i++) {
      for(j=0;j<3;j++) {
        for (k=0;k<4;k++) {
          symm[i+symnum][j][k]=-symm[i][j][k];
        }
      }
    }
    symnum*=2;
  }
  for(i=0;i<symnum;i++){
    for(j=0;j<3;j++){
      symm[i][j][0]-=floor(symm[i][j][0]);
    }
  }
}

SV_LocalFunction void CheckNonPosi(void)
{
  int i;
  for(i=0;i<n_atom;i++){
    if (IsNonPosi(orgAtom[i].uij)) orgAtom[i].warn=1;
  }
}

SV_Function int ffgets(FILE *fp)
{
char *a;
int i;
  do {
    a=fgets(fbuf,200,fp);
    if (a==NULL) return 0;
    for (i=0;fbuf[i]!=0 && fbuf[i]!='\n' && fbuf[i]!=';';i++);
    fbuf[i]=0;
  }while((fbuf[0]=='C' && fbuf[1]=='O' && fbuf[2]=='M'
          && fbuf[3]=='M') || fbuf[0]==0 || fbuf[0]=='#') ;
  return 1;
}

SV_Function void f_err(FILE *fp)
{
  fclose(fp);
  cmderr("File Read Error");
  return;
}

SV_LocalFunction void DecodeCell(void)
{
  Double V,cos_gammas;
  int i;
  if (cell[3]>PI) {
    /* Convert DEGREE -> RADIAN */
    cell[3]*=PI/180;
    cell[4]*=PI/180;
    cell[5]*=PI/180;
  }
  V = cell[0] * cell[1] * cell[2] * 
    sqrt(1 - pow(cos(cell[3]),2.0) 
         - pow(cos(cell[4]),2.0)-pow(cos(cell[5]),2.0) 
         + 2 * cos(cell[3]) * cos(cell[4]) * cos(cell[5]));
  for(i=0;i<3;i++){
    cell_cos[i]=cos(cell[3+i]);
    cell_sin[i]=sin(cell[3+i]);
  };
  for(i=0;i<3;i++) {
    cell_cos_s[i]=(cell_cos[i+1<3?i+1:i-2]*cell_cos[i+2<3?i+2:i-1]
                   -cell_cos[i])
      /(cell_sin[i+1<3?i+1:i-2]*cell_sin[i+2<3?i+2:i-1]);
    cell_sin_s[i]=sqrt(1.0-cell_cos_s[i]*cell_cos_s[i]);
  }
  for(i=0;i<3;i++) {
    rcell[i]=1/(cell[i]*cell_sin[i+1<3?i+1:i-2]*
                cell_sin_s[i+2<3?i+2:i-1]);
    rcell[i+3]=atan2(cell_sin[i],cell_cos[i]);
  };
  /*    cos_gammas = (cos(cell[3]) * cos(cell[4]) 
        - cos(cell[5])) / (sin(cell[3]) * sin(cell[4]));    */
  cos_gammas=cell_cos_s[2];
  MAT[0][0] = V/(cell[1]*cell[2]*sin(cell[3]));
  MAT[0][1] = 0;
  MAT[0][2] = 0;
  MAT[1][0] = -cell[0] * sin(cell[4]) * cos_gammas;
  MAT[1][1] = cell[1] * sin(cell[3]);
  MAT[1][2] = 0;
  MAT[2][0] = cell[0] * cos(cell[4]);
  MAT[2][1] = cell[1] * cos(cell[3]);
  MAT[2][2] = cell[2];
  {
    Float RMAT[3][3];
    inv_matrix(MAT,RMAT);
  }
}

SV_LocalFunction double DecodeFvarSub(double *fvar,double num)
{
    double xx;
    int yy;
    xx=fabs(num);
    yy=(xx+5)/10;
    xx-=yy*10;
    if (yy == 0) {
        return num;
    }
    if (yy == 1) {
        return xx;
    }
    if (num>0) {
        return fvar[yy-1]*xx;
    } else {
        return fvar[yy-1]*(1-xx);
    }
}

SV_LocalFunction void DecodeFvar(int n_fvar,double *fvar)
{
    int i,j;
    if (n_fvar==0) return;
    for(i=0;i<n_atom;i++){
        for(j=0;j<3;j++){
           orgAtom[i].loc[j]=DecodeFvarSub(fvar,orgAtom[i].loc[j]);
        }
        for(j=0;j<6;j++){
            orgAtom[i].uij[j]=DecodeFvarSub(fvar,orgAtom[i].uij[j]);
        }
        orgAtom[i].occ=DecodeFvarSub(fvar,orgAtom[i].occ);
    }
}

SV_LocalFunction void InitializeUnitCell(void)
{
  int i;
  CELL[0].loc[0]=0; CELL[0].loc[1]=0; CELL[0].loc[2]=0;
  CELL[1].loc[0]=0; CELL[1].loc[1]=0; CELL[1].loc[2]=0;
  CELL[2].loc[0]=0; CELL[2].loc[1]=0; CELL[2].loc[2]=0;
  CELL[3].loc[0]=0; CELL[3].loc[1]=0; CELL[3].loc[2]=0;
  CELL[4].loc[0]=0; CELL[4].loc[1]=0; CELL[4].loc[2]=0;
  sprintf(CELL[0].name," ");
  sprintf(CELL[1].name,"o");
  sprintf(CELL[2].name,"a");
  sprintf(CELL[3].name,"b");
  sprintf(CELL[4].name,"c");
  for(i=0;i<8;i++) {
    SYMCELL[i].atm=0;
    SYMCELL[i].sym=0;
    SYMCELL[i].sft[0]=0;
    SYMCELL[i].sft[1]=0;
    SYMCELL[i].sft[2]=0;
  };
  SYMCELL[0].atm=1;
  SYMCELL[1].atm=2;
  SYMCELL[2].atm=3;
  SYMCELL[4].atm=4;
  for(i=0;i<4;i++) {
    SYMCELL[i+4].sft[2]=1;
    SYMCELL[i*2+1].sft[0]=1;
    SYMCELL[i<2?i+6:i].sft[1]=1;
  }
  BCELL[0].atm[0]=0; BCELL[0].atm[1]=1;
  BCELL[1].atm[0]=2; BCELL[1].atm[1]=3;
  BCELL[2].atm[0]=4; BCELL[2].atm[1]=5;
  BCELL[3].atm[0]=6; BCELL[3].atm[1]=7;
  BCELL[4].atm[0]=0; BCELL[4].atm[1]=2;
  BCELL[5].atm[0]=1; BCELL[5].atm[1]=3;
  BCELL[6].atm[0]=4; BCELL[6].atm[1]=6;
  BCELL[7].atm[0]=5; BCELL[7].atm[1]=7;
  BCELL[8].atm[0]=0; BCELL[8].atm[1]=4;
  BCELL[9].atm[0]=1; BCELL[9].atm[1]=5;
  BCELL[10].atm[0]=2; BCELL[10].atm[1]=6;
  BCELL[11].atm[0]=3; BCELL[11].atm[1]=7;
  for(i=0;i<8;i++) {
    setup_symloc(SYMCELL+i,CELL);
  };
}

SV_Function int readfile2(void)
{
  FILE *fp;
  int i,oflag;
  int n_fvar=0;
  double fvar[20];
  Float prevUij=0;
  n_atom=0;
  symnum=0;
  oflag=1;

  symm[symnum][0][0]=0; symm[symnum][1][0]=0; symm[symnum][2][0]=0;
  symm[symnum][0][1]=1; symm[symnum][1][1]=0; symm[symnum][2][1]=0;
  symm[symnum][0][2]=0; symm[symnum][1][2]=1; symm[symnum][2][2]=0;
  symm[symnum][0][3]=0; symm[symnum][1][3]=0; symm[symnum][2][3]=1;
  symnum++;


/* Read Cell Parameter from "atoms.svx"   */
  if ((fp=ffopen("atoms.svx","r"))==NULL) {
    debug("Can't open atoms.sv\n");
    return 0;
  }
  while(ffgets(fp)) {
    char Cmd[5];
    Cmd[4]=0;
    strncpy(Cmd,fbuf,4);
    if (strcmp(Cmd,"TITL")==0) {
      strcpy(Title,fbuf+4);
    } else if (strcmp(Cmd,"CELL")==0) {
      if (sscanf(fbuf,"CELL %lf %lf %lf %lf %lf %lf",cell,cell+1,cell+2,
                 cell+3,cell+4,cell+5)!=6) { 
        PutMessage1("CELL line in atoms.svx is broken");
        fclose(fp);
        return 0;
      } else {
	DecodeCell();
      }
    } else if (strcmp(Cmd,"LATT")==0) {
      if (sscanf(fbuf,"LATT %d",&LATT)!=1) {
        PutMessage1("LATT line in atoms.svx is broken");
        fclose(fp);
        return 0;
      }
    } else if (strcmp(Cmd,"SYMM")==0) {
      if (sscanf(fbuf,"SYMM %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
        symm[symnum][0],symm[symnum][0]+1,symm[symnum][0]+2,symm[symnum][0]+3,
        symm[symnum][1],symm[symnum][1]+1,symm[symnum][1]+2,symm[symnum][1]+3,
        symm[symnum][2],symm[symnum][2]+1,symm[symnum][2]+2,symm[symnum][2]+3
                 ) == 12 ) {
	/* check symmetry +x,+y,+z and remove */
	int xx0,yy0,ff0=0;
	for(yy0=0;yy0<3;yy0++){
	  if (symm[symnum][yy0][0]!=0.0) { ff0=1; break; };
	  for(xx0=0;xx0<3;xx0++){
	    if (xx0==yy0) {
	      if (symm[symnum][yy0][xx0+1]!=1.0) { ff0=1; break; };
	    } else {
	      if (symm[symnum][yy0][xx0+1]!=0.0) { ff0=1; break; };
	    }
	  }
	}
        if (ff0!=0) symnum++;
      } else {
        PutMessage1("SYMM line in atoms.svx is broken");
        fclose(fp);
        return 0;
      }
    } else if (strcmp(Cmd,"ELEM")==0) {
      if (sscanf(fbuf,"ELEM %2s",sfac[n_sfac].name)!=1) {       
        PutMessage1("ELEM line in atoms.svx is broken");
        fclose(fp);
        return 0;
      } else {
        n_sfac++;
      }
    } else if (strcmp(Cmd,"ATOM")==0) {
      int j;
      for(j=0;j<6;j++)orgAtom[n_atom].uij[j]=0;
      orgAtom[n_atom].frag=orgAtom[n_atom].warn=0;
      if (sscanf(fbuf,"ATOM %4s %d %lf %lf %lf %lf %lf %lf %d %d %d",
             orgAtom[n_atom].name,
             &i,orgAtom[n_atom].loc,orgAtom[n_atom].loc+1,
             orgAtom[n_atom].loc+2,
             &(orgAtom[n_atom].occ),&(orgAtom[n_atom].uij[0]),
             &(orgAtom[n_atom].peak_h),
             &(orgAtom[n_atom].frag),&(orgAtom[n_atom].part),
             &(orgAtom[n_atom].afix)
             ) != 11) {
        PutMessage1("ATOM line in atoms.svx is broken");
        fclose(fp);
        return 0;
      } else {
        orgAtom[n_atom].atom_type=i;
	if (orgAtom[n_atom].uij[0]<-1.0) orgAtom[n_atom].uij[0]*=-prevUij;
        if (orgAtom[n_atom].uij[0]<0) orgAtom[n_atom].warn=1;
	if (strcmp(sfac[i-1].name,"H")!=0) {
	  prevUij=orgAtom[n_atom].uij[0];
	}
        if (orgAtom[n_atom].name[0]=='Q') {
          if (n_atom<10) oflag=0;
          SAT[n_atom].view_flag|=FLAG_OMIT*oflag;
        }
        n_atom++;
      }
    } else if (strcmp(Cmd,"UIJ ")==0) {
      if (sscanf(fbuf,"UIJ %lf %lf %lf %lf %lf %lf",
             orgAtom[n_atom-1].uij,orgAtom[n_atom-1].uij+1,
             orgAtom[n_atom-1].uij+2,orgAtom[n_atom-1].uij+3,
             orgAtom[n_atom-1].uij+4,orgAtom[n_atom-1].uij+5)!=6) {
        PutMessage1("Uij line in atoms.svx is broken");
        PutMessage1(fbuf);
        fclose(fp);
        return 0;
      } 
      if (strcmp(sfac[orgAtom[n_atom-1].atom_type-1].name,"H")!=0) {
	prevUij=get_ueq(orgAtom+n_atom-1);
      }
    } else if (strcmp(Cmd,"FILE")==0) {
    } else if (strcmp(Cmd,"FVAR")==0) {
        if (sscanf(fbuf,"FVAR %lf",&(fvar[n_fvar++]))!=1) {
            PutMessage1("FVAR line in atoms.svx is broken");
            fclose(fp);
            return 0;
        }
    } else {
      sprintf(fbuf,"Unknown keyword '%s' is found in atoms.svx",Cmd);
      PutMessage1(fbuf);
      fclose(fp);
      return 0;
    }
  } 
  fclose(fp);
  DecodeSfac();
  DecodeFvar(n_fvar,fvar);
  CheckNonPosi();
  DecodeSymmetry();
  InitializeUnitCell();
  orgAtom[n_atom].name[0]=0;
  BOND[n_bond=0].atm[0]=-1;
  n_satom=0;
  unpack_sub(oflag);
  return 1;
}
  
