/* Figure Output Module   "shelview.c"      -*- C -*- */

/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1998 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/

#define __shelview_c__
#define __subs_c_proc__
#define __shelplot_c_proc__

#include <math.h>
#include <ctype.h>
#include "def.h"

SV_LocalFunction void update_atom(void)
{
  int i,j,l;
  for(i=0;i<n_satom;i++) {
    setup_symloc(SAT+i,orgAtom);
  }
  l=0;
  if ((global_flag & GLOBALFLAG_GC)!=0) {
    avg[0]=avg[1]=avg[2]=0;
    for(i=0;i<n_satom;i++) {
      if ((SAT[i].view_flag & FLAG_OMIT)==0) {
        for(j=0;j<3;j++) {
          avg[j]+=SAT[i].vloc[j];
        }
        l++;
      }
    }
    if (l==0) l=1;
    for(i=0;i<3;i++) avg[i]=avg[i]/l;
  } else {
    for(i=0;i<3;i++) avg[i]=offst[i];
  }
}

SV_Function void calc_atoms(void)
{
  int i,j,k;
  Float zz;
  update_atom();
  for(i=0;i<n_satom;i++) {
    zz=0;
    for(k=0;k<3;k++){
      zz+=VMT[2][k]*(SAT[i].vloc[k]-avg[k]);
    }
    SAT[i].axy[2]=zz;
    zz=(ZVIEW==0)?vscale:vscale*ZVIEW/(-zz+ZVIEW);
    for(j=0;j<2;j++) {
      Double loc;
      loc=0;
      for(k=0;k<3;k++){
        loc+=VMT[j][k]*(SAT[i].vloc[k]-avg[k]);
      }
      SAT[i].axy[j]=(loc-shift[j])*zz+sshift[j];;
    }
  }
  for(i=0;i<8;i++) {
    zz=0;
    for(k=0;k<3;k++){
      zz+=VMT[2][k]*(SYMCELL[i].vloc[k]-avg[k]);
    }
    SYMCELL[i].axy[2]=zz;
    zz=(ZVIEW==0)?vscale:vscale*ZVIEW/(-zz+ZVIEW);
    for(j=0;j<2;j++) {
      Double loc;
      loc=0;
      for(k=0;k<3;k++){
        loc+=VMT[j][k]*(SYMCELL[i].vloc[k]-avg[k]);
      }
      SYMCELL[i].axy[j]=(loc-shift[j])*zz+sshift[j];
    }
  }
}
#ifdef IGraph
SV_Function void calc_loc(Float *vloc,Float *axy,int matnum,int orgnum)
{
  Float zz;
  Float ftmp[3];
  int j,k;
  zz=0;
  for(j=0;j<3;j++){
    ftmp[j]=irvmts[matnum][j][0]*vloc[0]+irvmts[matnum][j][1]*vloc[1]
      +irvmts[matnum][j][2]*vloc[2]+iorgs[orgnum][j]; 
  };
  for(k=0;k<3;k++){
    zz+=VMT[2][k]*(ftmp[k]-avg[k]);
  }
  axy[2]=zz;
  zz=(ZVIEW==0)?vscale:vscale*ZVIEW/(-zz+ZVIEW);
  for(j=0;j<2;j++) {
    Double loc;
    loc=0;
    for(k=0;k<3;k++){
      loc+=VMT[j][k]*(ftmp[k]-avg[k]);
    }
    axy[j]=(loc-shift[j])*zz+sshift[j];;
  }
}
#endif

SV_LocalFunction void DrawAtomByLine(S_AT *at1,void (*GCOLOR)(int),void (*GLINE)(Double,Double,Double,Double)){
  int col,mv;
  Float mx,my;
  mv=at1->view_flag;
  mx=at1->axy[0];
  my=at1->axy[1];
  if (orgAtom[at1->atm].warn!=0) {
    GCOLOR(2);
    GLINE(mx-0.3,my-0.3,mx-0.3,my+0.3);
    GLINE(mx-0.3,my+0.3,mx+0.3,my+0.3);
    GLINE(mx+0.3,my+0.3,mx+0.3,my-0.3);
    GLINE(mx+0.3,my-0.3,mx-0.3,my-0.3);
  }

  if (mv & FLAG_NOBOND){
    if (global_flag & GLOBALFLAG_BOND_COLOR) {
      GCOLOR(at1->color);
    } else {
      col=-at1->axy[2]/0.9+24;
      if (col<19) col=19;
      if (col>29) col=29;
      GCOLOR(col);
    }
    GLINE(mx+0.1,my+0.1,mx-0.1,my-0.1);
    GLINE(mx+0.1,my-0.1,mx-0.1,my+0.1);
  }

  if (mv & FLAG_SEL){
    GCOLOR(at1->color);
    GLINE(mx-0.05,my-0.05,mx-0.05,my+0.05);
    GLINE(mx-0.05,my+0.05,mx+0.05,my+0.05);
    GLINE(mx+0.05,my+0.05,mx+0.05,my-0.05);
    GLINE(mx+0.05,my-0.05,mx-0.05,my-0.05);
  }
}

SV_Function void shelview(void (*GCOLOR)(int),void (*GLINE)(Double,Double,Double,Double),void (*GPUTS)(Double,Double,char *))
{
  int i,j,k;

  /* Draw Scale (1 Angstrome) */
  GCOLOR(COLOR_NORMAL);
  GLINE(-4.8,-4.8,-4.8+vscale,-4.8);
  GLINE(-4.8,-4.8,-4.8,-4.8+vscale);
  /* Draw UnitCell */
  if (global_flag & GLOBALFLAG_CELL) {
    for(i=0;i<8;i++) {
      GPUTS(SYMCELL[i].axy[0],SYMCELL[i].axy[1],CELL[SYMCELL[i].atm].name);
    }
    for(i=0;i<12;i++){
      S_AT *j,*k;
      j=SYMCELL+BCELL[i].atm[0];
      k=SYMCELL+BCELL[i].atm[1];
      GLINE(j->axy[0],j->axy[1],k->axy[0],k->axy[1]);
    }
  }

  /* Draw Bond */
  for(i=0;i<n_satom;i++) {
    SAT[i].view_flag|=FLAG_NOBOND;
  }
  for(i=0;i<n_bond;i++) {
    S_AT *at1,*at2;
    int col;
    at1=SAT+BOND[i].atm[0];
    at2=SAT+BOND[i].atm[1];
    j=at1->view_flag;
    k=at2->view_flag;
    if ((j|k) & FLAG_OMIT) continue;
    at1->view_flag &= ~FLAG_NOBOND;
    at2->view_flag &= ~FLAG_NOBOND;
    if (global_flag & GLOBALFLAG_BOND_COLOR) {
      Float mx,my;
      mx=(at1->axy[0]+at2->axy[0])*.5;
      my=(at1->axy[1]+at2->axy[1])*.5;
      col=at1->color;
      GCOLOR(col);
      GLINE(mx,my,at1->axy[0],at1->axy[1]);
      col=at2->color;
      GCOLOR(col);
      GLINE(mx,my,at2->axy[0],at2->axy[1]);
      col=-(at1->axy[2]+at2->axy[2])/1.8+24;
      if (col<19) col=19;
      if (col>29) col=29;
    } else {
      col=-(at1->axy[2]+at2->axy[2])/1.8+24;
      if (col<19) col=19;
      if (col>29) col=29;
      GCOLOR(col);
      GLINE(at1->axy[0],at1->axy[1],at2->axy[0],at2->axy[1]);
    }

    if ((global_flag & GLOBALFLAG_LABEL) &&
        (global_flag & GLOBALFLAG_TMP_LABEL) &&
        (global_flag & GLOBALFLAG_BONDLEN)){
      GCOLOR(col-2);
      sprintf(fbuf,"%4.2f",BOND[i].len);
      GPUTS((at1->axy[0]+at2->axy[0])*.5,
            (at1->axy[1]+at2->axy[1])*.5,fbuf);
    }
  }

  /* Draw Atom */
  if ((global_flag & GLOBALFLAG_LABEL)!=0 && 
      (global_flag & GLOBALFLAG_TMP_LABEL)!=0)  {
    for(i=0;i<n_satom;i++) {
      j=SAT[i].view_flag;
      if (! (j & FLAG_OMIT)) {
	if (j & FLAG_NOLABEL)  {
	  DrawAtomByLine(SAT+i,GCOLOR,GLINE);
	} else {
	  GCOLOR((j & FLAG_SEL)?SAT[i].color+8:SAT[i].color);
	  if (OrgAtom(i)->warn!=0) GCOLOR(10);
	  if (global_flag & (GLOBALFLAG_LONGLABEL | GLOBALFLAG_DISPAFIX)) {
	    char *fb1,*fb2;
	    fb1=fbuf;
	    fb2=fbuf2;
	    if (global_flag & GLOBALFLAG_LONGLABEL) {
	      sprintf(fbuf,"_%d",EncodeAdc(SAT+i));
	    } else fbuf[0]=0;
	    if ((global_flag & GLOBALFLAG_DISPAFIX) && OrgAtom(i)->afix!=0) {
	      sprintf(fb2,"%s*%d",fbuf,OrgAtom(i)->afix);
	    } else {
	      fb2=fbuf;
	      fb1=fbuf2;
	    }
	    sprintf(fb1,"%s%s",OrgAtom(i)->name,fb2);
	    GPUTS(SAT[i].axy[0],SAT[i].axy[1],fb1);
	  } else {
	    GPUTS(SAT[i].axy[0],SAT[i].axy[1],OrgAtom(i)->name);
	  }
	}
      }
    }
  } else {
    /* Draw unconnected atom */
    for(i=0;i<n_satom;i++) {
      int v=SAT[i].view_flag;
      if ( !(v & FLAG_OMIT) ) {
	DrawAtomByLine(SAT+i,GCOLOR,GLINE);
      }
    }
  }
  
#ifdef IGraph
  if ((global_flag & GLOBALFLAG_IGRAPH)!=0 && 
      (global_flag & GLOBALFLAG_TMP_LABEL)!=0)  {
    Float v1[3],v2[3],v3[3],fx,fy,fz,fr;
    int ist,ied,nm,no;
    for(i=0;i<n_ilines;i++) {
      GCOLOR(ilines[i].col);
      calc_loc(ilines[i].vloc,v1,ilines[i].matnum,ilines[i].orgnum);
      calc_loc(ilines[i].vloc2,v2,ilines[i].matnum,ilines[i].orgnum);
      GLINE(v1[0],v1[1],v2[0],v2[1]);
    };
    for(i=0;i<n_iarcs;i++) {
      GCOLOR(iarcs[i].col);
      v3[0]=(fx=iarcs[i].vloc[0])+
        (fr=iarcs[i].r)*cos(PI/180*(ist=iarcs[i].st));
      v3[1]=(fy=iarcs[i].vloc[1])+fr*sin(PI/180*ist);
      v3[2]=(fz=iarcs[i].vloc[2]);
      calc_loc(v3,v1,nm=iarcs[i].matnum,no=iarcs[i].orgnum);
      ied=iarcs[i].ed;
      for(j=ist+1;j<=ied;j++) {
        v3[0]=fx+fr*cos(PI/180*j);
        v3[1]=fy+fr*sin(PI/180*j);
        v3[2]=fz;
        calc_loc(v3,v2,nm,no);
        GLINE(v1[0],v1[1],v2[0],v2[1]);
        v1[0]=v2[0];
        v1[1]=v2[1];
        v1[2]=v2[2];
      }
    }
  }
#endif
}

/*************** for All ****************/
SV_Function void grect(void (*GLINE)(Double,Double,Double,Double),Double x1,Double y1,Double x2,Double y2)
{
  GLINE(x1,y1,x1,y2);
  GLINE(x1,y2,x2,y2);
  GLINE(x2,y2,x2,y1);
  GLINE(x1,y1,x2,y1);
}
