/* Graphics Library Module   "shelplot.c"    -*- C -*-  */

/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1998 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/

/*
 * This file contains
 *    void ginit(void)
 *    void gopen(void)
 *    void gclose(void)
 *    void gcls(void)
 *    void gline(x,y,x2,y2,col)
 *    void gputs(x,y,str)
 *    void gputss(x,y,str)
 *    void ginit2(void)
 *
 * $Log$
 * Revision 1.1  2001/03/18 14:46:59  tnemoto
 * Initial revision
 *
 * Revision 1.2  1999/01/07 13:40:39  tnemoto
 * *** empty log message ***
 *
 * Revision 1.1.1.1  1998/10/29 14:10:42  tnemoto
 * Shelx Output Viewer 1.86
 *
 * Revision 1.1  1995/06/06  07:50:22  tnemoto
 * SV Source Tree
 *
 * Revision 1.2  1992/08/26  05:03:32  tnemoto
 * Xt 01
 *
 * Revision 1.1  1992/07/20  05:41:59  tnemoto
 * Initial revision
 *
 */
#define __shelplot_c__
#define __cmd_c_proc__
#define __stack_c_proc__

#ifdef HAVE_CONFIG_H
#define __CONFIG_H__
#include "../config.h"
#endif

#ifdef Xt
#define __xtmain_c_proc__
#define __xtcalls_c_proc__
#endif

#include "def.h"

SV_GlobalVariable int X11MODE,X11R3;
SV_Variable int COLORMODE;
SV_Variable int MAX_LINE;
/*********************************************************************
                   for unix   (Xt / Tek)
*********************************************************************/

#include <strings.h>

int reverse_flag;

SV_Function void ginit(void)
{
  int i;
#ifdef Xt
  static char *colorname[]={
    "black","blue","red","magenta","lime green","cyan","yellow","white"
  };
  static char *grayname[]={
    "gray0","gray6","gray12","gray18","gray24","gray30","gray36","gray42",
    "gray48","gray54","gray60","gray66","gray72","gray78","gray84","gray92"
  };
  static char dash[2]={5,5};
  static char dash2[2]={7,3};
#endif
  MAX_LINE=20;
  reverse_flag=0;
  if (X11MODE){
#ifdef Xt
    COLORMODE=1;
    if (DefaultDepth(disp,screen)<2) COLORMODE=0;
    font[0]=svdata.defaultFont;
    font[1]=svdata.reverseFont;
    for(i=0;i<33;i++) {
      gc_col[i]=XCreateGC(disp,DefaultRootWindow(disp),0,NULL);
      XSetFont(disp,gc_col[i],font[i>7 && i<16?1:0]);
      XSetFunction(disp,gc_col[i],GXcopy);
    };
    XSetFunction(disp,gc_col[32],GXxor);
    if (COLORMODE) {
      Colormap cmap;
      XColor xc1,xc2;
      cmap=DefaultColormap(disp,screen);
      XAllocNamedColor(disp,cmap,colorname[3],&xc1,&xc2);
      XSetForeground(disp,gc_col[32],xc1.pixel);
    } else {
      XSetForeground(disp,gc_col[32],1);
    }

    if (X11R3==0) {
      XSetDashes(disp, gc_col[32], 0, dash, 2);
      XSetLineAttributes(disp,gc_col[32],1,LineOnOffDash,CapButt,JoinMiter);
    };

    if (COLORMODE) {
      Colormap cmap;
      XColor xc1,xc2;
      cmap=DefaultColormap(disp,screen);
      for(i=0;i<16;i++) {
	XAllocNamedColor(disp,cmap,colorname[i&7],&xc1,&xc2);
	if (i>7){
	  XSetForeground(disp,gc_col[i],WhitePixel(disp,screen));
	  XSetBackground(disp,gc_col[i],xc1.pixel);
	} else {
	  XSetForeground(disp,gc_col[i],xc1.pixel);
	  XSetBackground(disp,gc_col[i],WhitePixel(disp,screen));
	}
      }
      for(i=16;i<32;i++) {
	XSetFont(disp,gc_col[i],font[0]);
	XAllocNamedColor(disp,cmap,grayname[i&15],&xc1,&xc2);
	XSetForeground(disp,gc_col[i],xc1.pixel);
	XSetBackground(disp,gc_col[i],WhitePixel(disp,screen));
      };
    } else { /* for monochrome mode */
      for(i=0;i<16;i++) {
	if (i>7){
	  XSetForeground(disp,gc_col[i],WhitePixel(disp,screen));
	  XSetBackground(disp,gc_col[i],BlackPixel(disp,screen));
	} else if (i<7) {
	  XSetForeground(disp,gc_col[i],BlackPixel(disp,screen));
	  XSetBackground(disp,gc_col[i],WhitePixel(disp,screen));
	} else {
	  XSetForeground(disp,gc_col[i],WhitePixel(disp,screen));
	  XSetBackground(disp,gc_col[i],WhitePixel(disp,screen));
        };
      }
      for(i=16;i<32;i++) {
	XSetFont(disp,gc_col[i],font[0]);
	XSetForeground(disp,gc_col[i],BlackPixel(disp,screen));
	XSetBackground(disp,gc_col[i],WhitePixel(disp,screen));
	if (i>24) {
	  XSetDashes(disp, gc_col[i], 0, dash2, 2);
	  XSetLineAttributes(disp,gc_col[i],1,LineOnOffDash,
			     CapButt,JoinMiter);
	}
      };
    }
#endif
  } else {
#ifdef TEK
    openpl();
    puts("\x1b[?38h\n");
    puts("\x1b%~0\n");
#endif
  }
}

SV_Function void ginit2(void)
{
#ifdef TEK
  if (!X11MODE) {
    openpl();
    erase();
    closepl();
    printf("\x1b[?38l\n");
  }
#endif
}

SV_Function void gflush(void)
{
#ifdef TEK
  if (!X11MODE) 
    {
      closepl();
      openpl();
    }
#endif
}

SV_Function void gcolor(int col)
{
  if (X11MODE) {
#ifdef Xt
    gc=gc_col[col];
    if (col>7 && col<16){
      reverse_flag=-1;
    } else {
      reverse_flag=0;
    }
#endif
  } else {
#ifdef TEK
    if (col>15) col=0;
    gclose();
    if (COLORMODE){
      switch(col & 7){
      case 0:
        printf("\x1bMT1\x1bML1\n");
        break;
      case 1:
        printf("\x1bMT5\x1bML5\n");
        break;
      case 2:
        printf("\x1bMT3\x1bML3\n");
        break;
      case 4:
        printf("\x1bMT4\x1bML4\n");
        break;
      case 7:
        printf("\x1bMT1\x1bML1\n");
        break;
      default:
        printf("\x1bMT1\x1bML1\n");
        break;
      }
    }
  gopen();
#endif /* TEK */
  }
}
  
SV_Function void gopen(void)
{
#ifdef TEK
  if (!X11MODE) openpl();
#endif
}

SV_Function void gcls(void)
{
  if (X11MODE) {
#ifdef Xt
    XStoreName(disp,XtWindow(toplevel),Title);
    XFillRectangle(disp,win,gc_col[7],0,0,WinWidth,WinHeight);
    /*	XClearWindow(disp,win);*/
#endif
  }  else {
#ifdef TEK
    erase();
#endif
  }
}

SV_Function void gclose(void)
{
#ifdef TEK
  if (!X11MODE)  closepl();
#endif
}

SV_Function void gputs(Double x,Double y,char *txt)
{
  if (!X11MODE)
    {
#ifdef TEK
        if (x>-5 && x<5 && y>-5 && y<5) {
	  move((int)(x*300+1500),(int)(y*300+1500));
	  label(txt);
	}
#endif
      }
  else
    {
#ifdef Xt
      if (reverse_flag) {
	XDrawImageString(disp,win,gc,(int)(x*WinScale+WinWidthHalf),
			 (int)(-y*WinScale+WinHeightHalf),
		    txt,strlen(txt));
      } else {
	XDrawString(disp,win,gc,(int)(x*WinScale+WinWidthHalf),
		    (int)(-y*WinScale+WinHeightHalf),txt,strlen(txt));
      }
#endif
    }
}

SV_Function void gputserr(char *txt)
{
#ifdef Xt
  if (X11MODE) {
/*
    XClearWindow(disp,win3);
    XDrawString(disp,win3,gc_col[0],10,20,
		 txt,strlen(txt));
    XFlush(disp);
*/
    if (txt!=NULL) {
      ErrorPopup(txt);
    } else {
      ErrorPopdown();
    }
  };
#endif
}

SV_Function void gputscmd(void)
{
#ifdef TEK
  if (!X11MODE) {
    move(100,100);
    label("Input Command : ");
  }
#endif
}

SV_Function void gputs2(char *txt)
{
#ifdef TEK
  static int x,y;
#endif
#ifdef Xt
  char scr[MaxWin2Buf];
#endif
  if (!X11MODE) 
    {
#ifdef TEK
      if (txt[0]==0) {x=3100;y=2800; return;}
      move(x,y);
      label(txt);
      y-=120;
#endif
    }
  else
    {
#ifdef Xt
      if (txt[0]==0) { win2buf[0]=0;  return ;}
      if (txt[0]=='#') {
	XtSetArg(arg[0],XtNstring,(XtArgVal)win2buf);
	XtSetValues(wwin2,arg,1);
	return;
      }
      if (strlen(txt)+strlen(win2buf)>MaxWin2Buf-100)
	sprintf(txt,"=== over flow ===");
      sprintf(scr,"%s\n%s",win2buf,txt);
      sprintf(win2buf,"%s",scr);
#endif
    }
}


SV_Function void gline(Double x,Double y,Double x2,Double y2)
{
  if (!X11MODE)
    {
#ifdef TEK
      if (x>-5 && x<5 && y>-5 && y<5 
	  && x2>-5 && x2<5 && y2>-5 && y2<5) {
	line((int)(x*300+1500),(int)(y*300+1500),
	     (int)(x2*300+1500),(int)(y2*300+1500));
      }
#endif
    }
  else 
    {
#ifdef Xt
      XDrawLine(disp,win,gc,(int)(x*WinScale+WinWidthHalf),
		(int)(-y*WinScale+WinHeightHalf),
		(int)(x2*WinScale+WinWidthHalf),
		(int)(-y2*WinScale+WinHeightHalf));
#endif
    }
}

SV_Function void gdisp_atom_stack_sub(int x,int y,char *txt)
{
#ifdef Xt
  XDrawString(disp,stack_pix,gc_col[0],x,y,txt,strlen(txt));
#endif
}

SV_Function void gdisp_atom_stack(void)
{
  char txt[50];
  int i;
  if (X11MODE) {
#ifdef Xt
    stack_pix=XtWindow(wstack);
    gcolor(0);
    XFillRectangle(disp,stack_pix,gc_col[7],0,0,220,130);
    gdisp_atom_stack_sub(0,20,"Atom");
    gdisp_atom_stack_sub(50,20,"Bond");
    gdisp_atom_stack_sub(100,20,"Angle");
    gdisp_atom_stack_sub(150,20,"Torsion");
    for(i=atom_stack_top>9?9:atom_stack_top;i>=0;i--){
      sprintf(txt,"%-4s",orgAtom[atom_stack_s[i]->atm].name);
      gdisp_atom_stack_sub(0,35+i*10,txt);
      if (i>0) {
	sprintf(txt,"%-6.2f",atom_st_bnd[i-1]);
	gdisp_atom_stack_sub(50,30+i*10,txt);
	if (i>1) {
	  sprintf(txt,"%-6.2f",atom_st_ang[i-2]);
	  gdisp_atom_stack_sub(100,25+i*10,txt);
	  if (i>2){
	    sprintf(txt,"%-6.2f",atom_st_tor[i-3]);
	    gdisp_atom_stack_sub(150,20+i*10,txt);
	  }
	}
      }
    }
    /*
    XtSetArg(arg[0],XtNbitmap,stack_pix);
    XtSetValues(wwin4,arg,1);
    */
#endif
  }
}

SV_Function void dispgraph(char *txt,int num) 
{
  if (X11MODE) {
#ifdef Xt
    XDispGraph(txt,num);
#endif
  } else {
    int i;
#ifdef TEK
    move(10,10);
#endif
    fprintf(stderr,"%s ",txt);
    for(i=0;i<num/2;i++) {
      fprintf(stderr,"*");
    }
    for(;i<50;i++){
      fprintf(stderr,"-");
    }
    fprintf(stderr,"\r");
  }
}
    
